<?php

namespace Drupal\wtotem\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\wtotem\lib\Common\API as WtotemAPI;

/**
 * Form with modal window.
 */
class WTotemPortsForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wtotem_ports_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['messages'] = [
      '#markup' => '<div id="wtotem_ports_form-messages"></div>',
      '#weight' => -100,
    ];

    $form['port'] = [
      '#type' => 'number',
      '#attributes' => [
        'placeholder' => $this->t('Type port number'),
      ],
      '#theme_wrappers' => [],
    ];

    $form['submit'] = [
      '#type' => 'button',
      '#value' => $this->t('Send'),
      '#attributes' => [
        'class' => ['button'],
      ],
      '#ajax' => [
        'callback' => '::ajaxSubmitCallback',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) { }

  /**
   * {@inheritdoc}
   */
  public function ajaxSubmitCallback(array &$form, FormStateInterface $form_state) {
    $ajax_response = new AjaxResponse();

    $host = WtotemAPI::siteInfo();
    $response = WtotemAPI::addIgnorePort($host['id'], $form_state->getValue('port'));

    if ($response['errors']) {
      $massage = '<div class="message error_message">' . $this->t('Failed') . '</div>';
    }
    else {
      $massage = '<div class="message success_message">' . $this->t('Success') . '</div>';
    }

    $ports = WtotemAPI::getAllPortsList($host['id']);

    $open_ports[] = [
      "#ports" => $ports,
      '#theme' => 'wtotem_open_ports',
    ];

    $ignore_ports[] = [
      "#ports" => $ports,
      '#theme' => 'wtotem_ignore_ports',
    ];

    $ajax_response->addCommand(new HtmlCommand('#open_ports_wrap', $open_ports));
    $ajax_response->addCommand(new HtmlCommand('#open_ports_modal_wrap', $open_ports));
    $ajax_response->addCommand(new HtmlCommand('#ignore_ports_wrap', $ignore_ports));

    $ajax_response->addCommand(new HtmlCommand('#wtotem_ports_form-messages', $massage));

    return $ajax_response;
  }

}
